package com.microsoft.mobileexperiences.bing.httpthreadpool;

import java.io.IOException;
import java.io.InputStream;

public class HttpResponseBase {
	public static int SUCCESS = 0; 
	public static int ERROR = -1; 
	public static int TIMEOUT = -2; 
	public static int EXCEPTION = -3;
	
	public static int NO_ERROR_CODE = 0;

	private int statusCode; // one of the values above. 
	private int errorCode; 
	private String errorDesc;

	
	public HttpResponseBase()
	{
		this(SUCCESS, NO_ERROR_CODE);
	}
	public HttpResponseBase(int statusCode, int errorCode)
	{
		this.statusCode = statusCode;
		this.errorCode = errorCode;
	}
	
	public HttpResponseBase(int statusCode, int errorCode, String errorDesc)
	{
		this.statusCode = statusCode;
		this.errorCode = errorCode;
		this.errorDesc = errorDesc;
	}
	
	public int getStatusCode()
	{
		return statusCode;
	}

	public int getErrorCode()
	{
		return errorCode;
	}
	
	public String getErrorDescription()
	{
		return errorDesc;
	}
	
	public void setStatusCode(int statusCode)
	{
		this.statusCode = statusCode;
	}
	
	public void setErrorCode(int errorCode)
	{
		this.errorCode = errorCode;
	}
	
	static public String getResponseString(InputStream responseStream)
	{
		byte[] buffer = getResponseByteArray(responseStream);
		return (buffer == null)? null : new String(buffer, 0, buffer.length);
	}
	
	//limit to 1 M bytes.
	static public byte[] getResponseByteArray(InputStream responseStream)
	{
		final int DEFAULT_TILE_BUFFER_SIZE = 1024;
        byte[] buffer = new byte[2 * DEFAULT_TILE_BUFFER_SIZE];            
        
        int bytesread = 0;
        int actual = 0;
        int totalLength = DEFAULT_TILE_BUFFER_SIZE;
        while (true) {                   
            try {
				actual = responseStream.read(buffer, bytesread, totalLength - bytesread);
			} catch (IOException e) {}
            if ( actual >= 0 && bytesread < 1024 * DEFAULT_TILE_BUFFER_SIZE)
            {
                bytesread += actual;
                if ( bytesread == totalLength)
                {
                    totalLength += DEFAULT_TILE_BUFFER_SIZE / 2;
                    byte[] newBuffer = new byte[totalLength];
                    System.arraycopy(buffer, 0, newBuffer, 0, bytesread);
                    buffer = newBuffer;                            
                }
            }
            else
                break;
        }  
        
        if (bytesread > 0)
        	return buffer;
        else 
        	return null;
        
    }
}
